@@ -31,10 +31,13 @@ def parseOUTCAR(OUTCAR_in,foutlog):
     fin=open(OUTCAR_in)
     l1=fin.readline()
     while l1:
-        if len(elems)==0 and "POTCAR:" in l1:
-            while len(l1.strip())>4:
+        if len(elems)==0:
+            while len(l1.strip())>4 and "POTCAR:" in l1:
                 l2=l1.split()
-                e2=l2[2].split("_")[0]
+                if "_" in l2[2]:
+                    e2=l2[2].split("_")[0]
+                else:
+                    e2=l2[2].strip()
                 elems.append(e2)
                 l1=fin.readline()
         if "direct lattice vectors" in l1:
