@@ -364,28 +364,25 @@ class DefectEqSolver:
         residArray0=func_resid(cp0)
         if residArray0 is None:
             return (None,None)
-        resid0=np.sqrt(np.square(residArray0).mean(axis=0))
-        vect_ms=0.0
-        for i1,v1 in enumerate(vect_grad):
-            vect_ms+=v1**2
-        vect_ms=vect_ms**0.5
-        vect1=[]
-        for i1,v1 in enumerate(vect_grad):
-            vect1.append(v1/vect_ms)
+        # resid0=np.max(np.fabs(residArray0))
+        # resid0=np.sqrt(np.square(residArray0).mean(axis=0))
+        resid0=np.linalg.norm(residArray0)
         cp1=[]
-        for i1,v1 in enumerate(vect1):
+        for i1,v1 in enumerate(vect_grad):
             cp1.append(cp0[i1]+step_size*v1)
-        i_loop=1
+        i_loop=0
         while i_loop<maxiter:
             residArray1=func_resid(cp1)
             if residArray1 is None:
                 return (None,None)
-            resid1=np.sqrt(np.square(residArray1).mean(axis=0))
+            resid1=np.max(np.fabs(residArray1))
+            # resid1=np.sqrt(np.square(residArray1).mean(axis=0))
+            resid1=np.linalg.norm(residArray1)
             if resid1>resid0:
                 break
             else:
                 cp2=[]
-                for i1,v1 in enumerate(vect1):
+                for i1,v1 in enumerate(vect_grad):
                     cp2.append(cp1[i1]+step_size*v1)
                 cp0=cp1
                 residArray0=residArray1
@@ -402,11 +399,13 @@ class DefectEqSolver:
             residArray2=func_resid(cp2)
             if residArray2 is None:
                 raise LineOptError("Minimization error!")
-                # return 0.0
-            resid2=np.sqrt(np.square(residArray2).mean(axis=0))
+            # resid2=np.max(np.fabs(residArray2))
+            # resid2=np.sqrt(np.square(residArray2).mean(axis=0))
+            resid2=np.linalg.norm(residArray2)
             return resid2
         try:
-            res3=optimize.minimize_scalar(func_resid_local,bracket=(0.0,1.0),method="golden",tol=tol_in,options={'xtol':tol_in})
+            # res3=optimize.minimize_scalar(func_resid_local,bracket=(0.0,1.0),method="golden",tol=tol_in,options={'xtol':tol_in})
+            res3=optimize.minimize_scalar(func_resid_local,bracket=(0.0,1.0),method="golden",tol=1e-5,options={'xtol':1e-10})
         except LineOptError:
             return(None,None)
         cp3=[]
@@ -415,7 +414,9 @@ class DefectEqSolver:
         residArray3=func_resid(cp3)
         if residArray3 is None:
             return (None,None)
-        resid3=np.sqrt(np.square(residArray3).mean(axis=0))
+        resid3=np.max(np.fabs(residArray3))
+        # resid3=np.sqrt(np.square(residArray3).mean(axis=0))
+        resid3=np.linalg.norm(residArray3)
         print("finished: opt_steady_line_minimization")
         return(cp3,residArray3)
 
@@ -426,11 +427,12 @@ class DefectEqSolver:
         if residArray2 is None:
             return (None,None)
         vgrad2=func_grad(residArray2)
-        resid2=np.sqrt(np.square(residArray2).mean(axis=0))
+        resid2=np.max(np.fabs(residArray2))
+        resid2=np.linalg.norm(residArray2)
         CGalgo="PR" ### PR may be best
         for iloop1 in range(maxiter):
             ###### CG
-            if iloop1==0:
+            if iloop1%(len(cp0)*2)==0:
                 cgvect2=copy.copy(vgrad2)
             else:
                 cgvect1=copy.copy(cgvect2)
@@ -450,12 +452,12 @@ class DefectEqSolver:
                 elif CGalgo=="HS":
                     for i1 in range(len(cp0)):
                         cgfact2+=vgrad2[i1]*(vgrad2[i1]-vgrad1[i1])
-                        cgfact1+=cgvect1[i1]*(vgrad2[i1]-vgrad1[i1])
+                        cgfact1-=cgvect1[i1]*(vgrad2[i1]-vgrad1[i1])
                 ## Dai-Yuan
                 elif CGalgo=="DY":
                     for i1 in range(len(cp0)):
                         cgfact2+=vgrad2[i1]**2
-                        cgfact1+=cgvect1[i1]*(vgrad2[i1]-vgrad1[i1])
+                        cgfact1-=cgvect1[i1]*(vgrad2[i1]-vgrad1[i1])
                 #######################
                 betaCG2=cgfact2/cgfact1
                 if CGalgo=="PR" and betaCG2<0.0:
@@ -464,9 +466,16 @@ class DefectEqSolver:
                 for i1 in range(len(cp0)):
                     cgvect2.append(vgrad2[i1]+betaCG2*cgvect1[i1])
             vgrad1=copy.copy(vgrad2)
+            cgvect2abs=0.0
+            for i1,v1 in enumerate(cgvect2):
+                cgvect2abs+=v1**2
+            cgvect2abs=cgvect2abs**0.5
+            cgvect3=[]
+            for i1,v1 in enumerate(cgvect2):
+                cgvect3.append(v1/cgvect2abs)
             fout.write(f" ----- New search starting ({iloop1+1}) -----\n")
-            fout.write(f" ----- Gradient for the line search = {cgvect2} -----\n")
-            (cp3,residArray2)=self.opt_steady_line_minimization(func_resid,cgvect2,cp2,delta_cp2,fout,tol,maxiter)
+            fout.write(f" ----- Gradient for the line search = {cgvect3} -----\n")
+            (cp3,residArray2)=self.opt_steady_line_minimization(func_resid,cgvect3,cp2,delta_cp2,fout,0.01*tol,maxiter)
             if residArray2 is None:
                 return (None,None)
             if damping_rate=="DividedByNsteps":
@@ -475,7 +484,8 @@ class DefectEqSolver:
                 delta_cp2*=damping_rate
             cp2=cp3
             vgrad2=func_grad(residArray2)
-            resid2=np.sqrt(np.square(residArray2).mean(axis=0))
+            # resid2=np.max(np.fabs(residArray2))
+            resid2=np.linalg.norm(residArray2)
             if resid2<tol:
                 break
         print("finished: opt_chempot_multi")
@@ -484,7 +494,7 @@ class DefectEqSolver:
     def opt_Natoms(self,temperature_in,chempots_in,Natoms_target_in,
                    opt_params_in,root_outfiles,plot_params_in):
         if not "opt_Natoms_tol" in opt_params_in.keys():
-            opt_params_in["opt_Natoms_tol"]=1e-7
+            opt_params_in["opt_Natoms_tol"]=1e-4
         if not "opt_Natoms_maxloop" in opt_params_in.keys():
             opt_params_in["opt_Natoms_maxloop"]=1000
         if not "opt_Natoms_output" in opt_params_in.keys():
@@ -537,7 +547,7 @@ class DefectEqSolver:
             str_outlabel=""
             for e1 in Natoms_elem_list:
                 str_outlabel+=f"   mu_{e1:<3}          |   Nat_{e1:<3}         |    delNat_{e1:<3}      |"
-            str_outlabel+=f"     residual       |"
+            str_outlabel+=f"    max(residual)   |"
             fout1.write(str_outlabel+"\n")
         if opt_params_in["opt_Natoms_output_every_step"]:
             opt_params_in["opt_Nsites_output_Edef_atlast"]=True
@@ -545,7 +555,9 @@ class DefectEqSolver:
             opt_params_in["opt_Nsites_output_Edef_atlast"]=False
         def output_local(cp_new,Nat_new,delNat_new):
             str1=""
-            resid_new=np.sqrt(np.square(delNat_new).mean(axis=0))
+            # resid_new=np.sqrt(np.square(delNat_new).mean(axis=0))
+            # resid_new=np.max(np.fabs(delNat_new))
+            resid_new=np.linalg.norm(delNat_new)
             for ie1,e1 in enumerate(Natoms_elem_list):
                 str1+=f" {cp_new[ie1]:17.14f} | {Nat_new[ie1]:17.14f} | {delNat_new[ie1]:18.11e} |"
             str1+=f" {resid_new:18.11e} |"
@@ -576,9 +588,10 @@ class DefectEqSolver:
         def func_grads_fixNatoms(residArray_in):
             jacob1=self.defects.get_Jacobian(Natoms_elem_list,temperature_in)
             vect_grads=np.zeros(len(Natoms_elem_list))
+            a1=1.0/(np.linalg.norm(residArray_in)**0.5)
             for ie1,e1 in enumerate(Natoms_elem_list):
                 for ie2,e2 in enumerate(Natoms_elem_list):
-                    vect_grads[ie1]-=2.0*residArray_in[ie2]*jacob1[ie2][ie1]
+                    vect_grads[ie1]-=a1*residArray_in[ie2]*jacob1[ie2][ie1]
             return vect_grads
         (Natoms_chempot_results_list,res_delNat)=self.opt_chempot_multi(
                 func_resid_fixNatoms,func_grads_fixNatoms,
